/* eslint-disable */
'use strict';

var gulp             = require('gulp'),
    autoprefixer     = require('autoprefixer'),
    del              = require('del'),
    changed          = require('gulp-changed'),
    cleanCSS         = require('gulp-clean-css'),
    environments     = require('gulp-environments'),
    fileinclude      = require('gulp-file-include'),
    imagemin         = require('gulp-imagemin'),
    notify           = require('gulp-notify'),
    plumber          = require('gulp-plumber'),
    postcss          = require('gulp-postcss'),
    rename           = require('gulp-rename'),
    sass             = require('gulp-sass'),
    sizereport       = require('gulp-sizereport'),
    sourcemaps       = require('gulp-sourcemaps'),
    stripDebug       = require('gulp-strip-debug'),
    svgmin           = require('gulp-svgmin'),
    svgstore         = require('gulp-svgstore'),
    uglify           = require('gulp-uglify-es').default,
    watch            = require('gulp-watch'),
    path             = require('path'),
    runSequence      = require('run-sequence'),
    buffer           = require('vinyl-buffer'),
    source           = require('vinyl-source-stream'),
    browserify       = require('browserify'),
    watchify         = require('watchify'),
    babelify         = require('babelify'),
    vueify           = require('vueify'),
    envify           = require('envify/custom'), // Use the envify custom module to specify environment variables
    browserSync      = require('browser-sync').create(),
    reload           = browserSync.reload;

var development = environments.development;
var production  = environments.production;

// Error handling
function errorAlert(error) {
    notify.onError({
        title: "Error",
        message: "Check your terminal"
    })(error);
    console.log(error.toString());
    this.emit('end');
}

// Clean dist folder
gulp.task('clean-dist', function () {
    return del('./dist/**', { force: true });
});

// Javascript
// Transpiling ES6 Modules to CommonJS Using Babel & Gulp

function buildJs(entryPaths, destName) {
    var entryObject = {
        entries: entryPaths,
        debug: true,
        cache: {},
        packageCache: {},
        // defining transforms here will avoid crashing your stream
        transform: [
            ['vueify', {
                sass: {
                    includePaths: [
                      './src/css'
                    ]
                }
            }],
            babelify,
            // Required in order to process node_modules files
            ['envify', {
                NODE_ENV: 'production',
                global: true
            }]
        ],
        sourceMaps: true
    }

    var b = production() ? browserify(entryObject) : watchify(browserify(entryObject));
    b.on('update', bundle);
    bundle();

    function bundle() {
        return b.bundle()
        .on('error', function(error) {
            notify.onError({
                title: "Error",
                message: "Check your terminal"
            })(error);
            console.log(error.toString());
            this.emit('end');
        })
        .pipe(source(destName))
        .pipe(buffer())
        .pipe(sourcemaps.init({loadMaps: true}))
        .pipe(production(stripDebug()))
        .pipe(production(uglify()))
        .pipe(sizereport({
            gzip: true
        }))
        .pipe(sourcemaps.write('./'))
        .pipe(gulp.dest('./dist/js'))
        .pipe(reload({stream: true}));
    }
};

gulp.task('build-common-js', function() {
    return buildJs(['./src/js/common.js'], 'common.js')
});

gulp.task('build-homepage-js', function() {
    return buildJs(['./src/js/homepage.js'], 'homepage.js')
});


// CSS
// Compile Sass files

gulp.task('build-styles', function () {
    return gulp.src('./src/**/main.scss')
    .pipe(plumber({errorHandler: errorAlert}))
    .pipe(sourcemaps.init())
    .pipe(sass({ includePaths: [ 'node_modules' ] }))
    .pipe(postcss([ autoprefixer() ]))
    .pipe(production(cleanCSS()))
    .pipe(sizereport({
        gzip: true
    }))
    .pipe(sourcemaps.write('./'))
    .pipe(gulp.dest('./dist/'))
    .pipe(reload({stream: true}));
});


// HTML
gulp.task('build-html', function() {
  return gulp.src(['./src/**/*.html', './src/**/*.inc'])
  .pipe(plumber({errorHandler: errorAlert}))
  .pipe(changed('./dist/', {hasChanged: changed.compareContents}))
  .pipe(fileinclude({
      prefix: '@@',
      basepath: '@file'
  }))
  .pipe(gulp.dest('./dist/'))
  .pipe(reload({stream: true}));
});

// Optimizing Images
gulp.task('build-img', function() {
    return gulp.src(['./src/img/**/*.+(png|gif|jpg)']) // , '!./src/img/dummy/**/*.+(png|gif|jpg)'
    .pipe(plumber({errorHandler: errorAlert}))
    .pipe(changed('./dist/img/'))
    .pipe(production(imagemin([
    	imagemin.gifsicle({interlaced: true}),
    	imagemin.jpegtran({progressive: true}),
    	imagemin.optipng({optimizationLevel: 5})
		])))
    .pipe(gulp.dest('./dist/img/'))
    .pipe(reload({stream: true}));
});


// Combine svg files into one with <symbol> elements.
gulp.task('build-svg', function() {
    return gulp.src(['./src/img/**/*.svg', '!./src/img/flags/tmp/**/*.svg'])
    .pipe(plumber({errorHandler: errorAlert}))
    .pipe(svgmin(function(file) {
        var prefix = path.basename(file.relative, path.extname(file.relative));
        return {
            plugins: [{
                cleanupIDs: {
                    prefix: prefix + '-',
                    minify: true
                }
            }]
        };
    }))
    .pipe(rename({prefix: 'icon-'}))
    .pipe(svgstore())
    .pipe(gulp.dest('./dist/img/'));
});

// Copy Fonts to dist folder
gulp.task('copy-font-files', function() {
    return gulp.src(['./src/fonts/*'])
    .pipe(plumber({errorHandler: errorAlert}))
    .pipe(gulp.dest('./dist/fonts/'))
    .pipe(reload({stream: true}));
});

// Copy Video to dist folder
gulp.task('copy-video-files', function() {
    return gulp.src(['./src/video/*'])
    .pipe(plumber({errorHandler: errorAlert}))
    .pipe(gulp.dest('./dist/video/'))
    .pipe(reload({stream: true}));
});

// Copy Favicon to dist folder
gulp.task('copy-favicon-folder', function() {
    return gulp.src(['./src/img/favicon/*'])
    .pipe(plumber({errorHandler: errorAlert}))
    .pipe(gulp.dest('./dist/img/favicon/'))
    .pipe(reload({stream: true}));
});

// Copy SVG's to dist folder
gulp.task('copy-svg-files', function() {
    return gulp.src(['./src/img/**/*.svg'])
    .pipe(plumber({errorHandler: errorAlert}))
    .pipe(changed('./dist/img/'))
    .pipe(gulp.dest('./dist/img/'))
    .pipe(reload({stream: true}));
});


// Static Server + watch files
gulp.task('serve', function() {
    browserSync.init({
        server: "./dist",
        index: "/html/index.html",
        notify: false,
        open: true,
        ghostMode: false,
        https: false,
        port: 8080,
        watchOptions : {
            ignored : 'node_modules/*',
            ignoreInitial : true
        }
    });

    gulp.watch('src/css/**/*.scss', {cwd: './'}, ['build-styles', reload]);
    gulp.watch(['src/**/*.html', 'src/**/*.inc'], {cwd: './'}, ['build-html', reload]);
    gulp.watch('src/img/**/*.+(png|gif|jpg|svg)', {cwd: './'}, ['build-img', 'build-svg', 'copy-svg-files', reload]);
});

// Run develop + watch
gulp.task('watch', function(callback) {
    runSequence(
        ['copy-font-files', 'copy-video-files', 'copy-favicon-folder', 'copy-svg-files'],
        ['build-img', 'build-svg'],
        ['build-homepage-js', 'build-common-js', 'build-styles'],
        ['build-html'],
        'serve',
    callback);
});

// Run develop
gulp.task('default', function(callback) {
    runSequence(
        ['copy-font-files', 'copy-video-files', 'copy-favicon-folder', 'copy-svg-files'],
        ['build-img', 'build-svg'],
        ['build-homepage-js', 'build-common-js', 'build-styles'],
    callback);
});

// Run build
gulp.task('build', function(callback) {
    runSequence(
        ['clean-dist'],
        ['copy-font-files', 'copy-favicon-folder', 'copy-svg-files'],
        ['build-img', 'build-svg'],
        ['build-homepage-js', 'build-common-js', 'build-styles'],
    callback);
});
