import { TweenMax, TimelineMax, Power4 } from 'gsap'
import Barba from 'barba.js'

const init = () => {
  if (window.barbaInitialized) return
  window.barbaInitialized = true

  const html = document.documentElement
  const body = document.body
  const siteSlope = document.querySelector('.site-slope--left polygon')

  const scriptCache = []
  Array.from(document.querySelectorAll('script')).forEach(el => scriptCache.push(el.src))

  Barba.Pjax.Dom.containerClass = 'valtech-container'
  Barba.Pjax.Dom.wrapperId = 'valtech-wrapper'
  Barba.Pjax.init()
  // Barba.Prefetch.init()

  // Slope in effect
  const slopeIn = () => {
    return new Promise(resolve => {
      const tl = new TimelineMax()
      tl.fromTo(siteSlope, 0.6, {
        attr: { points: '0,1080 0,1080 0,0 0,0' }
      }, {
        attr: { points: '1100,1080 0,1080 0,0 1280,0' },
        ease: Power4.easeOut
      }).to(siteSlope, 0.6, {
        attr: { points: '1920,1080 0,1080 0,0 1920,0' },
        ease: Power4.easeOut,
        onComplete: resolve
      }, '-=0.2')
    })
  }

  // Slope out effect
  const slopeOut = () => {
    return new Promise(resolve => {
      const tl = new TimelineMax()
      tl.to(siteSlope, 0.6, {
        attr: { points: '1920,1080 1280,1080 1100,0 1920,0' },
        delay: 1,
        ease: Power4.easeOut
      }).to(siteSlope, 0.6, {
        attr: { points: '1920,1080 1920,1080 1920,0 1920,0' },
        ease: Power4.easeOut,
        onComplete: resolve
      }, '-=0.2')
    })
  }

  // Bind in/out effects to Barba
  Barba.Pjax.getTransition = () => Barba.BaseTransition.extend({
    start () {
      return Promise.all([this.newContainerLoading, slopeIn()])
        .then(this.callPage.bind(this))
    },
    callPage () {
      // Close any open navigation
      const closenavigation = new CustomEvent('closenavigation')
      window.dispatchEvent(closenavigation)

      window.scrollTo(0, 0)
      slopeOut()
      this.done()
    }
  })

  // DarkSide theme
  Barba.BaseView.extend({
    namespace: 'dark-side',
    onEnterCompleted () {
      html.classList.remove('theme-light-side')
      html.classList.add('theme-dark-side')
    }
  }).init()

  // LightSide theme
  Barba.BaseView.extend({
    namespace: 'light-side',
    onEnterCompleted () {
      html.classList.remove('theme-dark-side')
      html.classList.add('theme-light-side')
    }
  }).init()

  // Fires when link clicked
  Barba.Dispatcher.on('linkClicked', () => {
    // Kill all ongoing animations
    const killanimations = new CustomEvent('killanimations')
    window.dispatchEvent(killanimations)

    // Kill all pending promises
    const killallpromises = new CustomEvent('killallpromises')
    window.dispatchEvent(killallpromises)
  })

  // Fires when new page is ready
  Barba.Dispatcher.on('newPageReady', () => {

  })

  // Fires when page transition is completed
  Barba.Dispatcher.on('transitionCompleted', () => {
    // Construct a DOM of the loaded HTML to query
    const dom = document.createElement('root')
    dom.innerHTML = Barba.Pjax.Dom.currentHTML

    // Collect all SCRIPT elements, render them and when all linked scripts are loaded, re-throw DOMContentLoaded event
    const scripts = Array.from(dom.querySelectorAll('script'))
    const promises = []

    scripts.forEach(el => {
      const script = document.createElement('script')
      if (el.src) {
        if (scriptCache.indexOf(el.src) === -1) { // Within runtime, load a unique script only once
          script.src = el.src
          promises.push(new Promise(resolve => {
            script.addEventListener('load', resolve)
            scriptCache.push(el.src)
          }))
        }
      } else {
        script.textContent = el.textContent
      }
      document.body.appendChild(script)
    })

    Promise.all(promises).then(() => setTimeout(() => {
      window.document.dispatchEvent(new Event('DOMContentLoaded', { bubbles: true, cancelable: true }))
    }, 0))
  })
}

// DOMContentLoaded event is fired when the initial HTML document has been completely loaded and parsed
document.addEventListener('DOMContentLoaded', init, { once: true })
